@@ -15,18 +15,12 @@ import threading
 from functools import partial
 
 from .ctypes_declarations import prototype, EXCEPTION_HANDLER_FUNCTYPE
-from .errors import WKBReadingError, WKTReadingError, TopologicalError, PredicateError
+from .errors import InvalidGeometryError, WKBReadingError, WKTReadingError, TopologicalError, PredicateError
 
 
 # Add message handler to this module's logger
 LOG = logging.getLogger(__name__)
 
-if sys.version_info[0] >= 3:
-    text_types = str
-else:
-    text_types = (str, unicode)
-
-
 # Find and load the GEOS and C libraries
 # If this ever gets any longer, we'll break it into separate modules
 
@@ -63,20 +57,39 @@ def load_dll(libname, fallbacks=None, mode=DEFAULT_MODE):
 
 _lgeos = None
 
+
+def exists_conda_env():
+    """Does this module exist in a conda environment?"""
+    return os.path.exists(os.path.join(sys.prefix, 'conda-meta'))
+
 if sys.platform.startswith('linux'):
-    # Test to see if we have a wheel repaired by 'auditwheel' containing its
-    # own libgeos_c
-    geos_whl_so = glob.glob(os.path.abspath(os.path.join(os.path.dirname(
-        __file__), '.libs/libgeos_c-*.so.*')))
-    if len(geos_whl_so) == 1:
-        _lgeos = CDLL(geos_whl_so[0])
+    # Test to see if we have a wheel repaired by auditwheel which contains its
+    # own libgeos_c. Note: auditwheel 3.1 changed the location of libs.
+    geos_whl_so = glob.glob(
+        os.path.abspath(os.path.join(os.path.dirname(__file__), ".libs/libgeos*.so*"))
+    ) or glob.glob(
+        os.path.abspath(
+            os.path.join(
+                os.path.dirname(__file__), "..", "Shapely.libs", "libgeos*.so*"
+            )
+        )
+    )
+
+    if len(geos_whl_so) > 0:
+        # We have observed problems with CDLL of libgeos_c not automatically
+        # loading the sibling c++ library since the change made by auditwheel
+        # 3.1, so we explicitly load them both.
+        geos_whl_so = sorted(geos_whl_so)
+        CDLL(geos_whl_so[0])
+        _lgeos = CDLL(geos_whl_so[-1])
         LOG.debug("Found GEOS DLL: %r, using it.", _lgeos)
+
     elif hasattr(sys, 'frozen'):
         geos_pyinstaller_so = glob.glob(os.path.join(sys.prefix, 'libgeos_c-*.so.*'))
-        if len(geos_pyinstaller_so) == 1:
+        if len(geos_pyinstaller_so) >= 1:
             _lgeos = CDLL(geos_pyinstaller_so[0])
             LOG.debug("Found GEOS DLL: %r, using it.", _lgeos)
-    elif os.getenv('CONDA_PREFIX', ''):
+    elif exists_conda_env():
         # conda package.
         _lgeos = CDLL(os.path.join(sys.prefix, 'lib', 'libgeos_c.so'))
     else:
@@ -85,81 +98,119 @@ if sys.platform.startswith('linux'):
             'libgeos_c.so',
         ]
         _lgeos = load_dll('geos_c', fallbacks=alt_paths)
-    # Necessary for environments with only libc.musl
-    c_alt_paths = [
-        'libc.musl-x86_64.so.1'
-    ]
-    free = load_dll('c', fallbacks=c_alt_paths).free
+
+    # ctypes.CDLL(None) internally calls dlopen(NULL), and as the dlopen
+    # manpage says, "If filename is NULL, then the returned handle is for the
+    # main program". This way we can let the linker do the work to figure out
+    # which libc Python is actually using.
+    free = CDLL(None).free
     free.argtypes = [c_void_p]
     free.restype = None
 
 elif sys.platform == 'darwin':
     # Test to see if we have a delocated wheel with a GEOS dylib.
-    geos_whl_dylib = os.path.abspath(os.path.join(os.path.dirname(
-        __file__), '.dylibs/libgeos_c.1.dylib'))
+    dylib_path = os.path.abspath(
+        os.path.join(os.path.dirname(__file__), ".dylibs/*.dylib")
+    )
+    LOG.debug("Formed path for globbing: dylib_path=%r", dylib_path)
 
-    if os.path.exists(geos_whl_dylib):
+    geos_whl_dylib = glob.glob(dylib_path)
+    LOG.debug("Globbed: geos_whl_dylib=%r", geos_whl_dylib)
+
+    if len(geos_whl_dylib) > 0:
         handle = CDLL(None)
         if hasattr(handle, "initGEOS_r"):
             LOG.debug("GEOS already loaded")
             _lgeos = handle
         else:
-            _lgeos = CDLL(geos_whl_dylib)
+            geos_whl_dylib = sorted(geos_whl_dylib)
+            CDLL(geos_whl_dylib[0])
+            _lgeos = CDLL(geos_whl_dylib[-1])
             LOG.debug("Found GEOS DLL: %r, using it.", _lgeos)
 
-    elif os.getenv('CONDA_PREFIX', ''):
+    elif hasattr(sys, "frozen"):
+        try:
+            # .app file from py2app
+            alt_paths = [
+                os.path.join(
+                    os.environ["RESOURCEPATH"], "..", "Frameworks", "libgeos_c.dylib"
+                )
+            ]
+        except KeyError:
+            alt_paths = [
+                # binary from pyinstaller
+                os.path.join(sys.executable, "libgeos_c.dylib"),
+                # .app from cx_Freeze
+                os.path.join(os.path.dirname(sys.executable), "libgeos_c.1.dylib"),
+            ]
+            if hasattr(sys, "_MEIPASS"):
+                alt_paths.append(os.path.join(sys._MEIPASS, "libgeos_c.1.dylib"))
+
+    elif exists_conda_env():
         # conda package.
         _lgeos = CDLL(os.path.join(sys.prefix, 'lib', 'libgeos_c.dylib'))
+
     else:
-        if hasattr(sys, 'frozen'):
-            try:
-                # .app file from py2app
-                alt_paths = [os.path.join(
-                    os.environ['RESOURCEPATH'], '..', 'Frameworks',
-                    'libgeos_c.dylib')]
-            except KeyError:
-                # binary from pyinstaller
-                alt_paths = [
-                    os.path.join(sys.executable, 'libgeos_c.dylib')]
-                if hasattr(sys, '_MEIPASS'):
-                    alt_paths.append(
-                        os.path.join(sys._MEIPASS, 'libgeos_c.1.dylib'))
-        else:
-            alt_paths = [
-                # The Framework build from Kyng Chaos
-                "/Library/Frameworks/GEOS.framework/Versions/Current/GEOS",
-                # macports
-                '/opt/local/lib/libgeos_c.dylib',
-                # homebrew
-                '/usr/local/lib/libgeos_c.dylib',
-            ]
+        alt_paths = [
+            # The Framework build from Kyng Chaos
+            "/Library/Frameworks/GEOS.framework/Versions/Current/GEOS",
+            # macports
+            "/opt/local/lib/libgeos_c.dylib",
+            # homebrew Intel
+            "/usr/local/lib/libgeos_c.dylib",
+            # homebrew Apple Silicon
+            "/opt/homebrew/lib/libgeos_c.dylib",
+        ]
         _lgeos = load_dll('geos_c', fallbacks=alt_paths)
 
-    free = load_dll('c').free
+    free = CDLL(None).free
     free.argtypes = [c_void_p]
     free.restype = None
 
 elif sys.platform == 'win32':
-    if os.getenv('CONDA_PREFIX', ''):
-        # conda package.
-        _lgeos = CDLL(os.path.join(sys.prefix, 'Library', 'bin', 'geos_c.dll'))
+    _conda_dll_path = os.path.join(sys.prefix, 'Library', 'bin', 'geos_c.dll')
+
+    if exists_conda_env() and os.path.exists(_conda_dll_path):
+        _lgeos = CDLL(_conda_dll_path)
+
     else:
-        try:
-            egg_dlls = os.path.abspath(
-                os.path.join(os.path.dirname(__file__), 'DLLs'))
-            if hasattr(sys, '_MEIPASS'):
-                wininst_dlls = sys._MEIPASS
-            elif hasattr(sys, "frozen"):
-                wininst_dlls = os.path.normpath(
-                    os.path.abspath(sys.executable + '../../DLLS'))
-            else:
-                wininst_dlls = os.path.abspath(os.__file__ + "../../../DLLs")
-            original_path = os.environ['PATH']
-            os.environ['PATH'] = "%s;%s;%s" % \
-                (egg_dlls, wininst_dlls, original_path)
-            _lgeos = load_dll("geos_c.dll")
-        except (ImportError, WindowsError, OSError):
-            raise
+        geos_whl_dll = glob.glob(
+            os.path.abspath(
+                os.path.join(
+                    os.path.dirname(__file__), "..", "Shapely.libs", "geos*.dll"
+                )
+            )
+        )
+
+        if len(geos_whl_dll) > 0:
+            geos_whl_dll = sorted(geos_whl_dll)
+            # CDLL(geos_whl_so[0])
+            _lgeos = CDLL(geos_whl_dll[-1])
+            LOG.debug("Found GEOS DLL: %r, using it.", _lgeos)
+
+        else:
+            try:
+                egg_dlls = os.path.abspath(
+                    os.path.join(os.path.dirname(__file__), "DLLs")
+                )
+                if hasattr(sys, "_MEIPASS"):
+                    wininst_dlls = sys._MEIPASS
+                elif hasattr(sys, "frozen"):
+                    wininst_dlls = os.path.normpath(
+                        os.path.abspath(sys.executable + "../../DLLS")
+                    )
+                else:
+                    wininst_dlls = os.path.abspath(os.__file__ + "../../../DLLs")
+                original_path = os.environ["PATH"]
+                os.environ["PATH"] = "%s;%s;%s" % (
+                    egg_dlls,
+                    wininst_dlls,
+                    original_path,
+                )
+                _lgeos = load_dll("geos_c.dll")
+
+            except (ImportError, WindowsError, OSError):
+                raise
 
         def free(m):
             try:
@@ -170,12 +221,13 @@ elif sys.platform == 'win32':
 
 elif sys.platform == 'sunos5':
     _lgeos = load_dll('geos_c', fallbacks=['libgeos_c.so.1', 'libgeos_c.so'])
-    free = CDLL('libc.so.1').free
+    free.restype = None
     free.argtypes = [c_void_p]
     free.restype = None
+
 else:  # other *nix systems
     _lgeos = load_dll('geos_c', fallbacks=['libgeos_c.so.1', 'libgeos_c.so'])
-    free = load_dll('c', fallbacks=['libc.so.6']).free
+    free = CDLL(None).free
     free.argtypes = [c_void_p]
     free.restype = None
 
@@ -184,9 +236,7 @@ def _geos_version():
     GEOSversion = _lgeos.GEOSversion
     GEOSversion.restype = c_char_p
     GEOSversion.argtypes = []
-    geos_version_string = GEOSversion()
-    if sys.version_info[0] >= 3:
-        geos_version_string = geos_version_string.decode('ascii')
+    geos_version_string = GEOSversion().decode('ascii')
     res = re.findall(r'(\d+)\.(\d+)\.(\d+)', geos_version_string)
     assert len(res) == 2, res
     geos_version = tuple(int(x) for x in res[0])
@@ -196,40 +246,38 @@ def _geos_version():
 geos_version_string, geos_version, geos_capi_version = _geos_version()
 
 
-# If we have the new interface, then record a baseline so that we know what
-# additional functions are declared in ctypes_declarations.
-if geos_version >= (3, 1, 0):
-    start_set = set(_lgeos.__dict__)
+# Record a baseline so that we know what additional functions are declared
+# in ctypes_declarations.
+start_set = set(_lgeos.__dict__)
 
 # Apply prototypes for the libgeos_c functions
 prototype(_lgeos, geos_version)
 
-# If we have the new interface, automatically detect all function
-# declarations, and declare their re-entrant counterpart.
-if geos_version >= (3, 1, 0):
-    end_set = set(_lgeos.__dict__)
-    new_func_names = end_set - start_set
+# Automatically detect all function declarations, and declare their
+# re-entrant counterpart.
+end_set = set(_lgeos.__dict__)
+new_func_names = end_set - start_set
 
-    for func_name in new_func_names:
-        new_func_name = "%s_r" % func_name
-        if hasattr(_lgeos, new_func_name):
-            new_func = getattr(_lgeos, new_func_name)
-            old_func = getattr(_lgeos, func_name)
-            new_func.restype = old_func.restype
-            if old_func.argtypes is None:
-                # Handle functions that didn't take an argument before,
-                # finishGEOS.
-                new_func.argtypes = [c_void_p]
-            else:
-                new_func.argtypes = [c_void_p] + old_func.argtypes
-            if old_func.errcheck is not None:
-                new_func.errcheck = old_func.errcheck
+for func_name in new_func_names:
+    new_func_name = "%s_r" % func_name
+    if hasattr(_lgeos, new_func_name):
+        new_func = getattr(_lgeos, new_func_name)
+        old_func = getattr(_lgeos, func_name)
+        new_func.restype = old_func.restype
+        if old_func.argtypes is None:
+            # Handle functions that didn't take an argument before,
+            # finishGEOS.
+            new_func.argtypes = [c_void_p]
+        else:
+            new_func.argtypes = [c_void_p] + list(old_func.argtypes)
+        if old_func.errcheck is not None:
+            new_func.errcheck = old_func.errcheck
 
-    # Handle special case.
-    _lgeos.initGEOS_r.restype = c_void_p
-    _lgeos.initGEOS_r.argtypes = \
-        [EXCEPTION_HANDLER_FUNCTYPE, EXCEPTION_HANDLER_FUNCTYPE]
-    _lgeos.finishGEOS_r.argtypes = [c_void_p]
+# Handle special case.
+_lgeos.initGEOS_r.restype = c_void_p
+_lgeos.initGEOS_r.argtypes = \
+    [EXCEPTION_HANDLER_FUNCTYPE, EXCEPTION_HANDLER_FUNCTYPE]
+_lgeos.finishGEOS_r.argtypes = [c_void_p]
 
 
 def make_logging_callback(func):
@@ -256,7 +304,7 @@ error_h = EXCEPTION_HANDLER_FUNCTYPE(error_handler)
 notice_h = EXCEPTION_HANDLER_FUNCTYPE(notice_handler)
 
 
-class WKTReader(object):
+class WKTReader:
 
     _lgeos = None
     _reader = None
@@ -275,10 +323,9 @@ class WKTReader(object):
 
     def read(self, text):
         """Returns geometry from WKT"""
-        if not isinstance(text, text_types):
+        if not isinstance(text, str):
             raise TypeError("Only str is accepted.")
-        if sys.version_info[0] >= 3:
-            text = text.encode()
+        text = text.encode()
         c_string = c_char_p(text)
         geom = self._lgeos.GEOSWKTReader_read(self._reader, c_string)
         if not geom:
@@ -290,77 +337,69 @@ class WKTReader(object):
         return geom_factory(geom)
 
 
-class WKTWriter(object):
+class WKTWriter:
 
     _lgeos = None
     _writer = None
 
     # Establish default output settings
-    defaults = {}
+    defaults = {
+        'trim': True,
+        'output_dimension': 3,
+    }
 
-    if geos_version >= (3, 3, 0):
+    # GEOS' defaults for methods without "get"
+    _trim = False
+    _rounding_precision = -1
+    _old_3d = False
 
-        defaults['trim'] = True
-        defaults['output_dimension'] = 3
+    @property
+    def trim(self):
+        """Trimming of unnecessary decimals (default: True)"""
+        return getattr(self, '_trim')
 
-        # GEOS' defaults for methods without "get"
-        _trim = False
-        _rounding_precision = -1
-        _old_3d = False
+    @trim.setter
+    def trim(self, value):
+        self._trim = bool(value)
+        self._lgeos.GEOSWKTWriter_setTrim(self._writer, self._trim)
 
-        @property
-        def trim(self):
-            """Trimming of unnecessary decimals (default: True)"""
-            return getattr(self, '_trim')
+    @property
+    def rounding_precision(self):
+        """Rounding precision when writing the WKT.
+        A precision of -1 (default) disables it."""
+        return getattr(self, '_rounding_precision')
 
-        @trim.setter
-        def trim(self, value):
-            self._trim = bool(value)
-            self._lgeos.GEOSWKTWriter_setTrim(self._writer, self._trim)
+    @rounding_precision.setter
+    def rounding_precision(self, value):
+        self._rounding_precision = int(value)
+        self._lgeos.GEOSWKTWriter_setRoundingPrecision(
+            self._writer, self._rounding_precision)
 
-        @property
-        def rounding_precision(self):
-            """Rounding precision when writing the WKT.
-            A precision of -1 (default) disables it."""
-            return getattr(self, '_rounding_precision')
+    @property
+    def output_dimension(self):
+        """Output dimension, either 2 or 3 (default)"""
+        return self._lgeos.GEOSWKTWriter_getOutputDimension(
+            self._writer)
 
-        @rounding_precision.setter
-        def rounding_precision(self, value):
-            self._rounding_precision = int(value)
-            self._lgeos.GEOSWKTWriter_setRoundingPrecision(
-                self._writer, self._rounding_precision)
+    @output_dimension.setter
+    def output_dimension(self, value):
+        self._lgeos.GEOSWKTWriter_setOutputDimension(
+            self._writer, int(value))
 
-        @property
-        def output_dimension(self):
-            """Output dimension, either 2 or 3 (default)"""
-            return self._lgeos.GEOSWKTWriter_getOutputDimension(
-                self._writer)
+    @property
+    def old_3d(self):
+        """Show older style for 3D WKT, without 'Z' (default: False)"""
+        return getattr(self, '_old_3d')
 
-        @output_dimension.setter
-        def output_dimension(self, value):
-            self._lgeos.GEOSWKTWriter_setOutputDimension(
-                self._writer, int(value))
-
-        @property
-        def old_3d(self):
-            """Show older style for 3D WKT, without 'Z' (default: False)"""
-            return getattr(self, '_old_3d')
-
-        @old_3d.setter
-        def old_3d(self, value):
-            self._old_3d = bool(value)
-            self._lgeos.GEOSWKTWriter_setOld3D(self._writer, self._old_3d)
+    @old_3d.setter
+    def old_3d(self, value):
+        self._old_3d = bool(value)
+        self._lgeos.GEOSWKTWriter_setOld3D(self._writer, self._old_3d)
 
     def __init__(self, lgeos, **settings):
         """Create WKT Writer
 
-        Note: writer defaults are set differently for GEOS 3.3.0 and up.
-        For example, with 'POINT Z (1 2 3)':
-
-            newer: POINT Z (1 2 3)
-            older: POINT (1.0000000000000000 2.0000000000000000)
-
-        The older formatting can be achieved for GEOS 3.3.0 and up by setting
+        Note: older formatting before GEOS 3.3.0 can be achieved by setting
         the properties:
             trim = False
             output_dimension = 2
@@ -391,17 +430,14 @@ class WKTWriter(object):
     def write(self, geom):
         """Returns WKT string for geometry"""
         if geom is None or geom._geom is None:
-            raise ValueError("Null geometry supports no operations")
+            raise InvalidGeometryError("Null geometry supports no operations")
         result = self._lgeos.GEOSWKTWriter_write(self._writer, geom._geom)
         text = string_at(result)
         lgeos.GEOSFree(result)
-        if sys.version_info[0] >= 3:
-            return text.decode('ascii')
-        else:
-            return text
+        return text.decode('ascii')
 
 
-class WKBReader(object):
+class WKBReader:
 
     _lgeos = None
     _reader = None
@@ -432,8 +468,7 @@ class WKBReader(object):
 
     def read_hex(self, data):
         """Returns geometry from WKB hex"""
-        if sys.version_info[0] >= 3:
-            data = data.encode('ascii')
+        data = data.encode('ascii')
         geom = self._lgeos.GEOSWKBReader_readHEX(
             self._reader, c_char_p(data), c_size_t(len(data)))
         if not geom:
@@ -445,7 +480,7 @@ class WKBReader(object):
         return geometry.base.geom_factory(geom)
 
 
-class WKBWriter(object):
+class WKBWriter:
 
     _lgeos = None
     _writer = None
@@ -515,7 +550,7 @@ class WKBWriter(object):
     def write(self, geom):
         """Returns WKB byte string for geometry"""
         if geom is None or geom._geom is None:
-            raise ValueError("Null geometry supports no operations")
+            raise InvalidGeometryError("Null geometry supports no operations")
         size = c_size_t()
         result = self._lgeos.GEOSWKBWriter_write(
             self._writer, geom._geom, pointer(size))
@@ -526,16 +561,13 @@ class WKBWriter(object):
     def write_hex(self, geom):
         """Returns WKB hex string for geometry"""
         if geom is None or geom._geom is None:
-            raise ValueError("Null geometry supports no operations")
+            raise InvalidGeometryError("Null geometry supports no operations")
         size = c_size_t()
         result = self._lgeos.GEOSWKBWriter_writeHEX(
             self._writer, geom._geom, pointer(size))
         data = string_at(result, size.value)
         lgeos.GEOSFree(result)
-        if sys.version_info[0] >= 3:
-            return data.decode('ascii')
-        else:
-            return data
+        return data.decode('ascii')
 
 
 # Errcheck functions for ctypes
@@ -555,10 +587,7 @@ def errcheck_just_free(result, func, argtuple):
     """Returns string from a C pointer"""
     retval = string_at(result)
     lgeos.GEOSFree(result)
-    if sys.version_info[0] >= 3:
-        return retval.decode('ascii')
-    else:
-        return retval
+    return retval.decode('ascii')
 
 
 def errcheck_null_exception(result, func, argtuple):
@@ -600,83 +629,14 @@ class LGEOSBase(threading.local):
             self.geos_handle = None
 
 
-class LGEOS300(LGEOSBase):
-    """Proxy for GEOS 3.0.0-CAPI-1.4.1
+class LGEOS330(LGEOSBase):
+    """Proxy for GEOS 3.3.0-CAPI-1.7.0
     """
-    geos_version = (3, 0, 0)
-    geos_capi_version = (1, 4, 0)
+    geos_version = (3, 3, 0)
+    geos_capi_version = (1, 7, 0)
 
     def __init__(self, dll):
-        super(LGEOS300, self).__init__(dll)
-        self.geos_handle = self._lgeos.initGEOS(notice_h, error_h)
-        keys = list(self._lgeos.__dict__.keys())
-        for key in keys:
-            setattr(self, key, getattr(self._lgeos, key))
-        self.GEOSFree = self._lgeos.free
-        # Deprecated
-        self.GEOSGeomToWKB_buf.errcheck = errcheck_wkb
-        self.GEOSGeomToWKT.errcheck = errcheck_just_free
-        self.GEOSRelate.errcheck = errcheck_null_exception
-        for pred in (
-                self.GEOSDisjoint,
-                self.GEOSTouches,
-                self.GEOSIntersects,
-                self.GEOSCrosses,
-                self.GEOSWithin,
-                self.GEOSContains,
-                self.GEOSOverlaps,
-                self.GEOSEquals,
-                self.GEOSEqualsExact,
-                self.GEOSRelatePattern,
-                self.GEOSisEmpty,
-                self.GEOSisValid,
-                self.GEOSisSimple,
-                self.GEOSisRing,
-                self.GEOSHasZ):
-            pred.errcheck = errcheck_predicate
-
-        self.methods['area'] = self.GEOSArea
-        self.methods['boundary'] = self.GEOSBoundary
-        self.methods['buffer'] = self.GEOSBuffer
-        self.methods['centroid'] = self.GEOSGetCentroid
-        self.methods['representative_point'] = self.GEOSPointOnSurface
-        self.methods['convex_hull'] = self.GEOSConvexHull
-        self.methods['distance'] = self.GEOSDistance
-        self.methods['envelope'] = self.GEOSEnvelope
-        self.methods['length'] = self.GEOSLength
-        self.methods['has_z'] = self.GEOSHasZ
-        self.methods['is_empty'] = self.GEOSisEmpty
-        self.methods['is_ring'] = self.GEOSisRing
-        self.methods['is_simple'] = self.GEOSisSimple
-        self.methods['is_valid'] = self.GEOSisValid
-        self.methods['disjoint'] = self.GEOSDisjoint
-        self.methods['touches'] = self.GEOSTouches
-        self.methods['intersects'] = self.GEOSIntersects
-        self.methods['crosses'] = self.GEOSCrosses
-        self.methods['within'] = self.GEOSWithin
-        self.methods['contains'] = self.GEOSContains
-        self.methods['overlaps'] = self.GEOSOverlaps
-        self.methods['equals'] = self.GEOSEquals
-        self.methods['equals_exact'] = self.GEOSEqualsExact
-        self.methods['relate'] = self.GEOSRelate
-        self.methods['difference'] = self.GEOSDifference
-        self.methods['symmetric_difference'] = self.GEOSSymDifference
-        self.methods['union'] = self.GEOSUnion
-        self.methods['intersection'] = self.GEOSIntersection
-        self.methods['relate_pattern'] = self.GEOSRelatePattern
-        self.methods['simplify'] = self.GEOSSimplify
-        self.methods['topology_preserve_simplify'] = \
-            self.GEOSTopologyPreserveSimplify
-
-
-class LGEOS310(LGEOSBase):
-    """Proxy for GEOS 3.1.0-CAPI-1.5.0
-    """
-    geos_version = (3, 1, 0)
-    geos_capi_version = (1, 5, 0)
-
-    def __init__(self, dll):
-        super(LGEOS310, self).__init__(dll)
+        super().__init__(dll)
         self.geos_handle = self._lgeos.initGEOS_r(notice_h, error_h)
         keys = list(self._lgeos.__dict__.keys())
         for key in [x for x in keys if not x.endswith('_r')]:
@@ -687,9 +647,15 @@ class LGEOS310(LGEOSBase):
                 setattr(self, key, attr)
             else:
                 setattr(self, key, getattr(self._lgeos, key))
-        if not hasattr(self, 'GEOSFree'):
-            # GEOS < 3.1.1
-            self.GEOSFree = self._lgeos.free
+
+        # GEOS 3.3.8 from homebrew has, but doesn't advertise
+        # GEOSPolygonize_full. We patch it in explicitly here.
+        key = 'GEOSPolygonize_full'
+        func = getattr(self._lgeos, key + '_r')
+        attr = partial(func, self.geos_handle)
+        attr.__name__ = func.__name__
+        setattr(self, key, attr)
+
         # Deprecated
         self.GEOSGeomToWKB_buf.func.errcheck = errcheck_wkb
         self.GEOSGeomToWKT.func.errcheck = errcheck_just_free
@@ -719,7 +685,9 @@ class LGEOS310(LGEOSBase):
                 self.GEOSisValid,
                 self.GEOSisSimple,
                 self.GEOSisRing,
-                self.GEOSHasZ):
+                self.GEOSHasZ,
+                self.GEOSisClosed,
+                self.GEOSCoveredBy):
             pred.func.errcheck = errcheck_predicate
 
         self.GEOSisValidReason.func.errcheck = errcheck_just_free
@@ -767,70 +735,9 @@ class LGEOS310(LGEOSBase):
         self.methods['simplify'] = self.GEOSSimplify
         self.methods['topology_preserve_simplify'] = \
             self.GEOSTopologyPreserveSimplify
+        self.methods['normalize'] = self.GEOSNormalize
         self.methods['cascaded_union'] = self.GEOSUnionCascaded
 
-
-class LGEOS311(LGEOS310):
-    """Proxy for GEOS 3.1.1-CAPI-1.6.0
-    """
-    geos_version = (3, 1, 1)
-    geos_capi_version = (1, 6, 0)
-
-    def __init__(self, dll):
-        super(LGEOS311, self).__init__(dll)
-
-
-class LGEOS320(LGEOS311):
-    """Proxy for GEOS 3.2.0-CAPI-1.6.0
-    """
-    geos_version = (3, 2, 0)
-    geos_capi_version = (1, 6, 0)
-
-    def __init__(self, dll):
-        super(LGEOS320, self).__init__(dll)
-
-        if geos_version >= (3, 2, 0):
-
-            def parallel_offset(geom, distance, resolution=16, join_style=1,
-                                mitre_limit=5.0, side='right'):
-                side = side == 'left'
-                if distance < 0:
-                    distance = abs(distance)
-                    side = not side
-                return self.GEOSSingleSidedBuffer(
-                    geom, distance, resolution, join_style, mitre_limit, side)
-
-            self.methods['parallel_offset'] = parallel_offset
-
-        self.methods['project'] = self.GEOSProject
-        self.methods['project_normalized'] = self.GEOSProjectNormalized
-        self.methods['interpolate'] = self.GEOSInterpolate
-        self.methods['interpolate_normalized'] = \
-            self.GEOSInterpolateNormalized
-        self.methods['buffer_with_style'] = self.GEOSBufferWithStyle
-        self.methods['hausdorff_distance'] = self.GEOSHausdorffDistance
-
-
-class LGEOS330(LGEOS320):
-    """Proxy for GEOS 3.3.0-CAPI-1.7.0
-    """
-    geos_version = (3, 3, 0)
-    geos_capi_version = (1, 7, 0)
-
-    def __init__(self, dll):
-        super(LGEOS330, self).__init__(dll)
-
-        # GEOS 3.3.8 from homebrew has, but doesn't advertise
-        # GEOSPolygonize_full. We patch it in explicitly here.
-        key = 'GEOSPolygonize_full'
-        func = getattr(self._lgeos, key + '_r')
-        attr = partial(func, self.geos_handle)
-        attr.__name__ = func.__name__
-        setattr(self, key, attr)
-
-        for pred in (self.GEOSisClosed,):
-            pred.func.errcheck = errcheck_predicate
-
         def parallel_offset(geom, distance, resolution=16, join_style=1,
                             mitre_limit=5.0, side='right'):
             if side == 'right':
@@ -839,12 +746,20 @@ class LGEOS330(LGEOS320):
                 geom, distance, resolution, join_style, mitre_limit)
 
         self.methods['parallel_offset'] = parallel_offset
+        self.methods['project'] = self.GEOSProject
+        self.methods['project_normalized'] = self.GEOSProjectNormalized
+        self.methods['interpolate'] = self.GEOSInterpolate
+        self.methods['interpolate_normalized'] = \
+            self.GEOSInterpolateNormalized
+        self.methods['buffer_with_style'] = self.GEOSBufferWithStyle
+        self.methods['hausdorff_distance'] = self.GEOSHausdorffDistance
         self.methods['unary_union'] = self.GEOSUnaryUnion
-        self.methods['is_closed'] = self.GEOSisClosed
         self.methods['cascaded_union'] = self.methods['unary_union']
+        self.methods['is_closed'] = self.GEOSisClosed
         self.methods['snap'] = self.GEOSSnap
         self.methods['shared_paths'] = self.GEOSSharedPaths
         self.methods['buffer_with_params'] = self.GEOSBufferWithParams
+        self.methods['covered_by'] = self.GEOSCoveredBy
 
 
 class LGEOS340(LGEOS330):
@@ -854,7 +769,7 @@ class LGEOS340(LGEOS330):
     geos_capi_version = (1, 8, 0)
 
     def __init__(self, dll):
-        super(LGEOS340, self).__init__(dll)
+        super().__init__(dll)
         self.methods['delaunay_triangulation'] = self.GEOSDelaunayTriangulation
         self.methods['nearest_points'] = self.GEOSNearestPoints
 
@@ -862,26 +777,49 @@ class LGEOS340(LGEOS330):
 class LGEOS350(LGEOS340):
     """Proxy for GEOS 3.5.0-CAPI-1.9.0
     """
+    geos_version = (3, 5, 0)
+    geos_capi_version = (1, 9, 0)
 
     def __init__(self, dll):
-        super(LGEOS350, self).__init__(dll)
+        super().__init__(dll)
         self.methods['clip_by_rect'] = self.GEOSClipByRect
+        self.methods['voronoi_diagram'] = self.GEOSVoronoiDiagram
 
 
-if geos_version >= (3, 5, 0):
+class LGEOS360(LGEOS350):
+    """Proxy for GEOS 3.6.0-CAPI-1.10.0
+    """
+    geos_version = (3, 6, 0)
+    geos_capi_version = (1, 10, 0)
+
+    def __init__(self, dll):
+        super().__init__(dll)
+        self.methods['minimum_clearance'] = self.GEOSMinimumClearance
+
+
+class LGEOS380(LGEOS360):
+    """Proxy for GEOS 3.8.0-CAPI-1.13.0"""
+
+    geos_version = (3, 8, 0)
+    geos_capi_version = (1, 13, 0)
+
+    def __init__(self, dll):
+        super().__init__(dll)
+        self.methods['make_valid'] = self.GEOSMakeValid
+
+
+if geos_version >= (3, 8, 0):
+    L = LGEOS380
+elif geos_version >= (3, 6, 0):
+    L = LGEOS360
+elif geos_version >= (3, 5, 0):
     L = LGEOS350
 elif geos_version >= (3, 4, 0):
     L = LGEOS340
 elif geos_version >= (3, 3, 0):
     L = LGEOS330
-elif geos_version >= (3, 2, 0):
-    L = LGEOS320
-elif geos_version >= (3, 1, 1):
-    L = LGEOS311
-elif geos_version >= (3, 1, 0):
-    L = LGEOS310
 else:
-    L = LGEOS300
+    raise ValueError('unexpected geos_version: ' + str(geos_version))
 
 lgeos = L(_lgeos)
 
